import { map } from 'rxjs/operator/map';
import { mergeAll } from 'rxjs/operator/mergeAll';
import { mergeMap } from 'rxjs/operator/mergeMap';
import { takeUntil } from 'rxjs/operator/takeUntil';
import { take } from 'rxjs/operator/take';
import { throttle } from 'rxjs/operator/throttle';
import { DeltaObservable } from './observables/DeltaObservable';

const parseOpts = (target, rootOrDeltaObservableClass, ...DeltaObservableClasses) => {
  let root = rootOrDeltaObservableClass;
  if (!root) {
    root = target;
  } else if (root === DeltaObservable || root.prototype instanceof DeltaObservable) {
    DeltaObservableClasses.unshift(root);
    root = target;
  }
  return [target, root, DeltaObservableClasses];
};

export class Scrolljack extends DeltaObservable {
  static scrollStart(...args) {
    const [target, root, DeltaObservableClasses] = parseOpts(...args);
    const sources = DeltaObservableClasses.map(DeltaClass => {
      var _context;

      return (_context = DeltaClass.scrollStart(target).hijack(), throttle).call(_context, () => DeltaClass.scrollStop(root).hijack());
    });

    if (sources.length > 1) {
      return this.merge(...sources);
    } else {
      return sources[0];
    }
  }

  static scroll(...args) {
    var _context2;

    return (_context2 = this.scrollWindow(...args), mergeAll).call(_context2);
  }

  static scrollWindow(...args) {
    const [target, root, DeltaObservableClasses] = parseOpts(...args);
    const sources = DeltaObservableClasses.map(DeltaClass => {
      var _context3;

      return (_context3 = DeltaClass.scrollStart(target).hijack(), map).call(_context3, v => {
        var _context4;

        return (_context4 = DeltaClass.scroll(root).hijack(), takeUntil).call(_context4, DeltaClass.scrollStop(root).hijack());
      });
    });

    if (sources.length > 1) {
      return this.merge(...sources);
    } else {
      return sources[0];
    }
  }

  static scrollStop(...args) {
    const [target, root, DeltaObservableClasses] = parseOpts(...args);
    const sources = DeltaObservableClasses.map(DeltaClass => {
      var _context5;

      return (_context5 = DeltaClass.scrollStart(target).hijack(), mergeMap).call(_context5, () => {
        var _context6;

        return (_context6 = DeltaClass.scrollStop(root).hijack(), take).call(_context6, 1);
      });
    });

    if (sources.length > 1) {
      return this.merge(...sources);
    } else {
      return sources[0];
    }
  }
}

export default Scrolljack;