var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { KinematicUpdater } from './KinematicUpdater';
import { computeNextValue } from './computeNextValue';
import { parseXOpts, parseYOpts } from './parseOpts';

class MomentumUpdater extends KinematicUpdater {
  _initSpring(spring) {
    spring.velocity = 0;
    spring.lastDelta = 0;
  }

  _updateFrameSpring(value, spring) {
    spring.lastDelta = spring.toDelta(value);
    spring.velocity = spring.toVelocity(value);
  }

  _computeNextSpring(value, spring) {
    // If we've not stopped yet, just pass the value through.
    if (!this.stopped) return value;

    const t = value.deltaT / 1000;
    let { lastDelta, velocity, stiffness: K, damping: B, precision: P } = spring;

    // Compute the result of the current frame.
    let [nd, nv] = computeNextValue(lastDelta, 0, velocity, t, K, B, P);

    return _extends({}, value, spring.fromDelta(nd), spring.fromVelocity(nv));
  }

  _shouldGenerateNextSpring(spring) {
    return spring.velocity !== 0;
  }
}

export const momentumX = (opts, ...args) => new MomentumUpdater([parseXOpts(opts)], ...args);

export const momentumY = (opts, ...args) => new MomentumUpdater([parseYOpts(opts)], ...args);

export const momentum = (opts, ...args) => new MomentumUpdater([parseXOpts(opts), parseYOpts(opts)], ...args);

momentum.x = momentumX;
momentum.y = momentumY;

export default momentum;