'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Keyboard = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _symbolObservable = require('symbol-observable');

var _symbolObservable2 = _interopRequireDefault(_symbolObservable);

var _DeltaObservable2 = require('./observables/DeltaObservable');

var _DeltaOperator = require('./operators/DeltaOperator');

var _EmulatedKeyboardEventObservable = require('./observables/EmulatedKeyboardEventObservable');

var _events = require('./events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Keyboard = exports.Keyboard = function (_DeltaObservable) {
  _inherits(Keyboard, _DeltaObservable);

  function Keyboard(target) {
    var event = arguments.length <= 1 || arguments[1] === undefined ? _events.KEY_MOVE : arguments[1];

    _classCallCheck(this, Keyboard);

    if (typeof target[_symbolObservable2.default] === 'function') {
      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Keyboard).call(this, target));
    } else {
      var source = _EmulatedKeyboardEventObservable.EmulatedKeyboardEventObservable.create(target, event, ignoreUnmappedKeys);
      source.operator = new _DeltaOperator.DeltaOperator(keyCodeToScrollDelta, keyCodeToScrollVelocity);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Keyboard).call(this, source));
    }
    return _possibleConstructorReturn(_this);
  }

  _createClass(Keyboard, null, [{
    key: 'scrollStart',
    value: function scrollStart(target) {
      return _get(Object.getPrototypeOf(Keyboard), 'scrollStart', this).call(this, target, _events.KEY_START);
    }
  }, {
    key: 'scrollStop',
    value: function scrollStop(target) {
      return _get(Object.getPrototypeOf(Keyboard), 'scrollStop', this).call(this, target, _events.KEY_END);
    }
  }]);

  return Keyboard;
}(_DeltaObservable2.DeltaObservable);

exports.default = Keyboard;


var LINE_DELTA = 40;
var PAGE_DELTA = 583;

var CODES = ['Space', 'PageUp', 'PageDown', 'End', 'Home', 'ArrowLeft', 'ArrowUp', 'ArrowRight', 'ArrowDown'];

var KEY_CODES_2_CODES = {
  32: 'Space',
  33: 'PageUp',
  34: 'PageDown',
  35: 'End',
  36: 'Home',
  37: 'ArrowLeft',
  38: 'ArrowUp',
  39: 'ArrowRight',
  40: 'ArrowDown'
};

var keyCodeToScrollVelocity = function keyCodeToScrollVelocity(event, lastEvent, deltaT) {
  var velocityX = void 0;
  var velocityY = void 0;

  if (!lastEvent) {
    velocityX = 0;
    velocityY = 0;
  } else {
    var _keyCodeToScrollDelta = keyCodeToScrollDelta(event);

    var deltaX = _keyCodeToScrollDelta.deltaX;
    var deltaY = _keyCodeToScrollDelta.deltaY;

    var t = deltaT / 1000;
    velocityX = deltaX / t;
    velocityY = deltaY / t;
  }

  return { velocityX: velocityX, velocityY: velocityY };
};

var keyCodeToScrollDelta = function keyCodeToScrollDelta(_ref) {
  var code = _ref.code;
  var keyCode = _ref.keyCode;

  if (!code) code = KEY_CODES_2_CODES[keyCode];
  switch (code) {
    case 'PageUp':
    case 'Home':
      return { deltaX: 0, deltaY: -PAGE_DELTA };
    case 'Space':
    case 'PageDown':
    case 'End':
      return { deltaX: 0, deltaY: PAGE_DELTA };
    case 'ArrowUp':
      return { deltaX: 0, deltaY: -LINE_DELTA };
    case 'ArrowDown':
      return { deltaX: 0, deltaY: LINE_DELTA };
    case 'ArrowLeft':
      return { deltaX: -LINE_DELTA, deltaY: 0 };
    case 'ArrowRight':
      return { deltaX: LINE_DELTA, deltaY: 0 };
  }
};

var ignoreUnmappedKeys = function ignoreUnmappedKeys(_ref2) {
  var code = _ref2.code;
  var keyCode = _ref2.keyCode;
  var ctrlKey = _ref2.ctrlKey;
  var shiftKey = _ref2.shiftKey;
  var altKey = _ref2.altKey;
  var metaKey = _ref2.metaKey;

  if (ctrlKey || shiftKey || altKey || metaKey) return false;
  if (!code) code = KEY_CODES_2_CODES[keyCode];
  if (!CODES.includes(code)) return false;
  return true;
};