'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEventSourcePool = undefined;

var _symbolObservable = require('symbol-observable');

var _symbolObservable2 = _interopRequireDefault(_symbolObservable);

var _HijackableEventObservable = require('../observables/HijackableEventObservable');

var _filter = require('rxjs/operator/filter');

var _share = require('rxjs/operator/share');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createEventSourceFactory = function createEventSourceFactory(EmulatorClass) {
  return function (target, eventType) {
    var _context;

    var eventSource = void 0;
    if (typeof target[_symbolObservable2.default] === 'function') {
      eventSource = _filter.filter.call(target, function (_ref) {
        var type = _ref.type;
        return type === eventType;
      });
    } else {
      eventSource = _HijackableEventObservable.HijackableEventObservable.create(target, eventType).lift(new EmulatorClass());
    }
    return (_context = eventSource, _share.share).call(_context);
  };
};

var createEventSourcePool = exports.createEventSourcePool = function createEventSourcePool(EmulatorClass) {
  var eventSourceFactory = createEventSourceFactory(EmulatorClass);
  var eventTargets = new WeakMap();

  var getter = function getter(target, eventType) {
    var realEventType = arguments.length <= 2 || arguments[2] === undefined ? eventType : arguments[2];

    // If this target has no sources yet, create one now.
    if (!eventTargets.has(target)) {
      // Create a map of event types to sources for this target.
      var eventTypes = new Map();
      eventTargets.set(target, eventTypes);

      // Create a source for the 'real' event type.
      var eventSource = eventSourceFactory(target, realEventType);
      // Pool our 'real' event source.
      eventTypes.set(realEventType, eventSource);

      // If the 'real' event type is not the same as the event type we want,
      // create a source for the type we want (it will be the 'real' source)
      // filtered for the type we want.
      if (realEventType !== eventType) {
        eventSource = eventSourceFactory(eventSource, eventType);
        // Pool our event source.
        eventTypes.set(eventType, eventSource);
      }

      // Return the created event source.
      return eventSource;
    } else {
      // Get the map of pooled event types for this target.
      var _eventTypes = eventTargets.get(target);

      // If we have no source for this event type on this target, create one now.
      if (!_eventTypes.has(eventType)) {
        var _eventSource = void 0;

        // If we had no 'real' event source on this target, create one now.
        if (!_eventTypes.has(realEventType)) {
          _eventSource = eventSourceFactory(target, realEventType);
          // Pool our 'real' event source.
          _eventTypes.set(realEventType, _eventSource);
        } else {
          // Get the 'real' event source.
          _eventSource = _eventTypes.get(realEventType);
        }

        // If the 'real' event type is not the same as the event type we want,
        // create a source for the type we want (it will be the 'real' source)
        // filtered for the type we want.
        if (realEventType !== eventType) {
          _eventSource = eventSourceFactory(_eventSource, eventType);
          // Pool our event source.
          _eventTypes.set(eventType, _eventSource);
        }

        // Return the created event source.
        return _eventSource;
      } else {
        // Return the pooled event source.
        return _eventTypes.get(eventType);
      }
    }
  };

  getter.get = getter;
  return getter;
};

exports.default = createEventSourcePool;