'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DeltaGeneratorObservable = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Observable2 = require('rxjs/Observable');

var _from2 = require('rxjs/observable/from');

var _animationFrame = require('rxjs/scheduler/animationFrame');

var _anchor = require('../updaters/anchor');

var _marked = [createDeltaGenerator].map(regeneratorRuntime.mark);

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DeltaGeneratorObservable = exports.DeltaGeneratorObservable = function (_Observable) {
  _inherits(DeltaGeneratorObservable, _Observable);

  function DeltaGeneratorObservable(startValue, endValue, updater, scheduler) {
    _classCallCheck(this, DeltaGeneratorObservable);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(DeltaGeneratorObservable).call(this));

    _this.startValue = startValue;
    _this.endValue = endValue;
    _this.updater = updater;
    _this.scheduler = scheduler;
    return _this;
  }

  _createClass(DeltaGeneratorObservable, [{
    key: '_subscribe',
    value: function _subscribe(subscriber) {
      var startValue = this.startValue;
      var endValue = this.endValue;
      var updater = this.updater;
      var scheduler = this.scheduler;

      if (typeof updater === 'function') updater = updater();

      if (endValue) {
        // We subtract our target delta from the updater's net delta so that
        // it ends up generating that amount of delta in the original orientation
        // as it attempts to bring the netDelta back to 0.
        updater.updateFrame(_extends({}, endValue, {
          deltaX: startValue.deltaX - endValue.deltaX,
          deltaY: startValue.deltaY - endValue.deltaY
        }));
      }

      return DeltaGeneratorObservable.from(updater, scheduler, startValue)._subscribe(subscriber);
    }
  }], [{
    key: 'create',
    value: function create(startValue, endValue) {
      var updater = arguments.length <= 2 || arguments[2] === undefined ? _anchor.anchor : arguments[2];
      var scheduler = arguments.length <= 3 || arguments[3] === undefined ? _animationFrame.animationFrame : arguments[3];

      return new DeltaGeneratorObservable(startValue, endValue, updater, scheduler);
    }
  }, {
    key: 'from',
    value: function from() {
      var updater = arguments.length <= 0 || arguments[0] === undefined ? _anchor.anchor : arguments[0];
      var scheduler = arguments.length <= 1 || arguments[1] === undefined ? _animationFrame.animationFrame : arguments[1];
      var initialValue = arguments[2];

      return (0, _from2.from)(createDeltaGenerator(scheduler, updater, scheduler.now(), initialValue), scheduler);
    }
  }]);

  return DeltaGeneratorObservable;
}(_Observable2.Observable);

exports.default = DeltaGeneratorObservable;


var F = 1000 / 60; // Default frame rate

function createDeltaGenerator(scheduler, updater, time) {
  var lastValue = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  var _lastValue, _lastValue$deltaX, deltaX, _lastValue$deltaY, deltaY, now, deltaT, t, droppedFrames, droppedValue, i, newValue;

  return regeneratorRuntime.wrap(function createDeltaGenerator$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          if (!updater.shouldGenerateNext()) {
            _context.next = 22;
            break;
          }

          _lastValue = lastValue;
          _lastValue$deltaX = _lastValue.deltaX;
          deltaX = _lastValue$deltaX === undefined ? 0 : _lastValue$deltaX;
          _lastValue$deltaY = _lastValue.deltaY;
          deltaY = _lastValue$deltaY === undefined ? 0 : _lastValue$deltaY;
          now = scheduler.now();
          deltaT = now - time;

          // If it's too soon to update, drop an update.

          if (!(deltaT < 1)) {
            _context.next = 10;
            break;
          }

          return _context.abrupt('continue', 0);

        case 10:

          // If it seems like we've dropped a lot of frames, its probably because
          // this process was backgrounded (switched tabs), so we should restart.
          if (deltaT > F * 10) {
            deltaT = F;
          }

          t = deltaT / 1000;

          // Calculate the number of frames that have been 'dropped' since the
          // last update. Dropped frames are updates that should've happened within
          // a window of time, but didn't, usually because of jank, normalizing
          // optimizations, or other delays introduced by the user/browser/runtime.

          droppedFrames = Math.floor(deltaT / F);


          if (droppedFrames) {
            droppedValue = {
              deltaX: deltaX / (droppedFrames + 1),
              deltaY: deltaY / (droppedFrames + 1),
              deltaT: F
            };

            droppedValue.velocityX = deltaX / t;
            droppedValue.velocityY = deltaY / t;

            // Subtract dropped frames' time deltas from the original time delta
            // to get the time deltas for just the latest frame.
            deltaT = deltaT - droppedValue.deltaT * droppedFrames;
            t = deltaT / 1000;

            // Apply the dropped frame deltas to the destination.
            for (i = 0; i < droppedFrames; i++) {
              droppedValue = updater.computeNext(droppedValue);
              updater.catchFrame(droppedValue);
              // Subtract dropped frames' deltas from the original deltas
              // to get the deltas for just the latest frame. We do this in the loop
              // instead of outside of it because the updater may adjust the values
              // on each iteration.
              deltaX -= droppedValue.deltaX;
              deltaY -= droppedValue.deltaY;
            }
          }

          newValue = {
            deltaX: deltaX,
            deltaY: deltaY,
            deltaT: deltaT,
            velocityX: deltaX / t,
            velocityY: deltaY / t
          };


          time = now;
          lastValue = updater.computeNext(newValue);
          updater.updateFrame(lastValue);

          _context.next = 20;
          return lastValue;

        case 20:
          _context.next = 0;
          break;

        case 22:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
}