'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DeltaObservable = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _symbolObservable = require('symbol-observable');

var _symbolObservable2 = _interopRequireDefault(_symbolObservable);

var _Observable2 = require('rxjs/Observable');

var _merge2 = require('rxjs/operator/merge');

var _DeltaGeneratorObservable = require('./DeltaGeneratorObservable');

var _HijackableEventObservable = require('./HijackableEventObservable');

var _DeltaOperator = require('../operators/DeltaOperator');

var _RectOperator = require('../operators/RectOperator');

var _MomentumOperator = require('../operators/MomentumOperator');

var _AnchorOperator = require('../operators/AnchorOperator');

var _AccumulationOperator = require('../operators/AccumulationOperator');

var _HijackOperator = require('../operators/HijackOperator');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DEFAULT_VALUE = Object.freeze({
  deltaT: 0,
  deltaX: 0,
  deltaY: 0,
  velocityX: 0,
  velocityY: 0
});

var DeltaObservable = exports.DeltaObservable = function (_Observable) {
  _inherits(DeltaObservable, _Observable);

  function DeltaObservable(target, event, predicate, computeDelta, computeVelocity) {
    _classCallCheck(this, DeltaObservable);

    if (!target) {
      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(DeltaObservable).call(this));
    } else if (typeof target[_symbolObservable2.default] === 'function') {
      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(DeltaObservable).call(this));

      _this.source = target[_symbolObservable2.default]();
    } else {
      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(DeltaObservable).call(this));

      _this.source = _HijackableEventObservable.HijackableEventObservable.create(target, event, predicate);
      _this.source.operator = new _DeltaOperator.DeltaOperator(computeDelta, computeVelocity);
    }
    return _possibleConstructorReturn(_this);
  }

  _createClass(DeltaObservable, [{
    key: 'lift',
    value: function lift(operator) {
      var observable = new this.constructor(this);
      observable.operator = operator;
      return observable;
    }
  }, {
    key: 'accumulate',
    value: function accumulate(initialValue) {
      initialValue = this.constructor.createValue(initialValue);
      return this.lift(new _AccumulationOperator.AccumulationOperator(initialValue));
    }
  }, {
    key: 'hijack',
    value: function hijack(predicate) {
      return this.lift(new _HijackOperator.HijackOperator(predicate));
    }
  }, {
    key: 'rect',
    value: function rect(bounds, latestValueOrSourceOrScheduler, scheduler) {
      var latestValueOrSource = latestValueOrSourceOrScheduler;
      if (latestValueOrSource.x != null || latestValueOrSource.y != null) {
        latestValueOrSource = this.constructor.createValue({
          deltaX: latestValueOrSource.x,
          deltaY: latestValueOrSource.y
        });
      } else if ((0, _utils.hasDelta)(latestValueOrSource)) {
        latestValueOrSource = this.constructor.createValue(latestValueOrSource);
      }

      return this.lift(new _RectOperator.RectOperator(bounds, latestValueOrSource, scheduler));
    }
  }, {
    key: 'momentum',
    value: function momentum(optsOrLatestSource, latestSourceOrScheduler, scheduler) {
      return this.lift(new _MomentumOperator.MomentumOperator(optsOrLatestSource, latestSourceOrScheduler, scheduler));
    }
  }, {
    key: 'anchor',
    value: function anchor(optsOrLatestSource, latestSourceOrScheduler, scheduler) {
      return this.lift(new _AnchorOperator.AnchorOperator(optsOrLatestSource, latestSourceOrScheduler, scheduler));
    }

    // FIXME: This isn't an operator, so should it be here?

  }], [{
    key: 'createValue',
    value: function createValue(opts) {
      opts = _extends({}, DEFAULT_VALUE, opts);
      var _opts = opts;
      var deltaT = _opts.deltaT;
      var deltaX = _opts.deltaX;
      var deltaY = _opts.deltaY;
      var velocityX = _opts.velocityX;
      var velocityY = _opts.velocityY;

      return { deltaT: deltaT, deltaX: deltaX, deltaY: deltaY, velocityX: velocityX, velocityY: velocityY };
    }
  }, {
    key: 'from',
    value: function from(target) {
      return new this(target);
    }
  }, {
    key: 'merge',
    value: function merge() {
      return this.from(_merge2.mergeStatic.apply(undefined, arguments));
    }
  }, {
    key: 'scrollStart',
    value: function scrollStart(target, event) {
      return new this(target, event);
    }
  }, {
    key: 'scrollStop',
    value: function scrollStop(target, event) {
      return new this(target, event);
    }
  }, {
    key: 'scroll',
    value: function scroll(target, event) {
      return new this(target, event);
    }
  }, {
    key: 'scrollTo',
    value: function scrollTo(endValue, updater, scheduler) {
      var startValue = this.createValue();
      endValue = this.createValue(endValue);
      return new this(_DeltaGeneratorObservable.DeltaGeneratorObservable.create(startValue, endValue, updater, scheduler));
    }
  }]);

  return DeltaObservable;
}(_Observable2.Observable);

exports.default = DeltaObservable;