'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.KinematicSubscriber = exports.KinematicOperator = undefined;

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _symbolObservable = require('symbol-observable');

var _symbolObservable2 = _interopRequireDefault(_symbolObservable);

var _Subscriber2 = require('rxjs/Subscriber');

var _DeltaGeneratorObservable = require('../observables/DeltaGeneratorObservable');

var _Updater = require('../updaters/Updater');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var KinematicOperator = exports.KinematicOperator = function () {
  function KinematicOperator(optsOrLatestSource, latestSourceOrScheduler, scheduler) {
    _classCallCheck(this, KinematicOperator);

    var opts = optsOrLatestSource;
    var latestSource = latestSourceOrScheduler;
    var initialValue = void 0;

    if (opts && typeof opts.schedule === 'function') {
      scheduler = opts;
      opts = null;
      latestSource = null;
    } else if (opts && typeof opts[_symbolObservable2.default] === 'function') {
      scheduler = latestSource;
      latestSource = opts;
      opts = null;
    } else if ((0, _utils.isDeltaLike)(opts)) {
      scheduler = latestSource;
      initialValue = opts;
      latestSource = null;
      opts = null;
    } else if ((0, _utils.isDeltaLike)(latestSource)) {
      initialValue = latestSource;
      latestSource = null;
    } else if (latestSource && typeof latestSource.schedule === 'function') {
      scheduler = latestSource;
      latestSource = null;
    }

    this.opts = opts;
    this.latestSource = latestSource;
    this.initialValue = initialValue;
    this.scheduler = scheduler;
  }

  _createClass(KinematicOperator, [{
    key: '_getUpdater',
    value: function _getUpdater() {
      return new _Updater.Updater();
    }
  }, {
    key: 'call',
    value: function call(subscriber, source) {
      var opts = this.opts;
      var latestSource = this.latestSource;
      var initialValue = this.initialValue;
      var scheduler = this.scheduler;

      var updater = this._getUpdater(opts);
      return source._subscribe(new KinematicSubscriber(subscriber, updater, initialValue, latestSource, scheduler));
    }
  }]);

  return KinematicOperator;
}();

exports.default = KinematicOperator;

var KinematicSubscriber = exports.KinematicSubscriber = function (_Subscriber) {
  _inherits(KinematicSubscriber, _Subscriber);

  function KinematicSubscriber(destination, updater, initialValue, latestSource, scheduler) {
    _classCallCheck(this, KinematicSubscriber);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(KinematicSubscriber).call(this, destination));

    _this.scheduler = scheduler;
    _this.updater = updater;
    _this.latestSource = latestSource;

    if (latestSource) {
      _this.add(latestSource.subscribe(function (value) {
        return _this._updateFrame(value);
      }));
    }

    if (initialValue) {
      _this._updateFrame(initialValue);
    }
    return _this;
  }

  _createClass(KinematicSubscriber, [{
    key: '_next',
    value: function _next(value) {
      this._start();
      value = this._computeNext(value);
      // Only update the frame value if we don't have a source for the latest value.
      if (!this.latestSource) this._updateFrame(value);
      _get(Object.getPrototypeOf(KinematicSubscriber.prototype), '_next', this).call(this, value);
    }
  }, {
    key: '_start',
    value: function _start() {
      if (this.updater.stopped) {
        this.updater.start();
      }
    }
  }, {
    key: '_stop',
    value: function _stop() {
      this.updater.stop();
    }
  }, {
    key: '_computeNext',
    value: function _computeNext(value) {
      return this.updater.computeNext(value);
    }
  }, {
    key: '_updateFrame',
    value: function _updateFrame(value) {
      if ((0, _utils.hasDelta)(value)) {
        this.updater.updateFrame(value);
      }
    }
  }, {
    key: '_complete',
    value: function _complete() {
      var _this2 = this;

      var subscriber = this.destination;
      var updater = this.updater;
      var scheduler = this.scheduler;


      this._stop();

      this.add(_DeltaGeneratorObservable.DeltaGeneratorObservable.from(updater, scheduler).subscribe({
        next: function next(value) {
          subscriber.next(value);
        },
        complete: function complete() {
          _get(Object.getPrototypeOf(KinematicSubscriber.prototype), '_complete', _this2).call(_this2);
        }
      }));
    }
  }]);

  return KinematicSubscriber;
}(_Subscriber2.Subscriber);