"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
// If we just reuse the same array wrapper for each computation, we can save
// an allocation. The caveat is that return values inside the array need to be
// immediately destructured by the caller, rather than keeping a reference
// to the array.
var computedNext = [];

// A mass-spring-damper system directly inspired by (stolen from) react-motion.
var computeNextValue = exports.computeNextValue = function computeNextValue(value, endValue, velocity, time, stiffness, damping, precision) {
  // Calculate the acceleration due to net forces from
  // current velocity plus the restorative spring force.
  var spring = -stiffness * (value - endValue);
  var damper = -damping * velocity;
  var a = spring + damper;

  // Calculate the actual velocity and offset after net forces have been applied.
  var newVelocity = velocity + a * time;
  var newValue = value + newVelocity * time;

  // Round our value to PRECISION.
  if (Math.abs(newVelocity) < precision && Math.abs(newValue - endValue) < precision) {
    newValue = endValue;
    newVelocity = 0;
  }

  // Return the new net delta and the new velocity.
  computedNext[0] = newValue;
  computedNext[1] = newVelocity;

  return computedNext;
};