'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseYOpts = exports.parseXOpts = exports.parseOptsArray = exports.parseOpts = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _iterall = require('iterall');

var K = 170; // Default stiffness
var B = 26; //  Default damping
var P = 0.01; // Default precision

var toDeltaFromDeltaX = function toDeltaFromDeltaX(_ref) {
  var deltaX = _ref.deltaX;
  return deltaX;
};
var toDeltaXFromDelta = function toDeltaXFromDelta(delta) {
  return { deltaX: delta };
};

var toDeltaFromDeltaY = function toDeltaFromDeltaY(_ref2) {
  var deltaY = _ref2.deltaY;
  return deltaY;
};
var toDeltaYFromDelta = function toDeltaYFromDelta(delta) {
  return { deltaY: delta };
};

var toVelocityFromVelocityX = function toVelocityFromVelocityX(_ref3) {
  var velocityX = _ref3.velocityX;
  return velocityX;
};
var toVelocityXFromVelocity = function toVelocityXFromVelocity(velocity) {
  return { velocityX: velocity };
};

var toVelocityFromVelocityY = function toVelocityFromVelocityY(_ref4) {
  var velocityY = _ref4.velocityY;
  return velocityY;
};
var toVelocityYFromVelocity = function toVelocityYFromVelocity(velocity) {
  return { velocityY: velocity };
};

var parseOpts = exports.parseOpts = function parseOpts(optsLike) {
  if (typeof optsLike.toDelta !== 'function') {
    throw new Error('A `toDelta` option is required.');
  }

  if (typeof optsLike.fromDelta !== 'function') {
    throw new Error('A `fromDelta` option is required.');
  }

  if (typeof optsLike.toVelocity !== 'function') {
    throw new Error('A `toVelocity` option is required.');
  }

  if (typeof optsLike.fromVelocity !== 'function') {
    throw new Error('A `fromVelocity` option is required.');
  }

  return _extends({
    stiffness: K,
    damping: B,
    precision: P,
    minNetDelta: 0,
    maxNetDelta: 0
  }, optsLike);
};

var parseOptsArray = exports.parseOptsArray = function parseOptsArray(opts) {
  if ((0, _iterall.isCollection)(opts)) {
    opts = Array.from(opts);
  } else {
    opts = [opts];
  }
  return opts.map(parseOpts);
};

var parseXOpts = exports.parseXOpts = function parseXOpts(opts) {
  return parseOpts(_extends({
    toDelta: toDeltaFromDeltaX,
    fromDelta: toDeltaXFromDelta,
    toVelocity: toVelocityFromVelocityX,
    fromVelocity: toVelocityXFromVelocity,
    minNetDelta: opts && opts.x || 0,
    maxNetDelta: (opts && opts.x || 0) + (opts && opts.width || 0)
  }, opts));
};

var parseYOpts = exports.parseYOpts = function parseYOpts(opts) {
  return parseOpts(_extends({
    toDelta: toDeltaFromDeltaY,
    fromDelta: toDeltaYFromDelta,
    toVelocity: toVelocityFromVelocityY,
    fromVelocity: toVelocityYFromVelocity,
    minNetDelta: opts && opts.y || 0,
    maxNetDelta: (opts && opts.y || 0) + (opts && opts.height || 0)
  }, opts));
};